==> paradox-dblize-1.1.10.tgz: package/test/bigquery.js <==
'use strict';

const chai = require('chai');
const mocha = require('mocha');
const should = require('should');
const asyncp = require('paradox-asyncp');

const Dblize = require('../index');
const _ = require('lodash');
const Promise = require('bluebird');

describe('Bigquery', function() {
    this.timeout(10 * 1000);

    let conn = null;
    let TestModel = null;
    let PartitionTestModel = null;

    /* -------------------------------------------------------------------------
     * Before
     * ---------------------------------------------------------------------- */

    before(function() {
        conn = new Dblize({
            adapter: 'bigquery',
            credentials: {
                projectId: process.env.CRED_PROJECTID,
                keyFilename: process.env.CRED_KEYFILE,
            },
            namespace: 'test',
            sync: {
                force: false,
            }
            // forcePartition: true,
            // emptyThrowException: false,
        });
    });

    /* -------------------------------------------------------------------------
     * Create
     * ---------------------------------------------------------------------- */

    const validateRecord = instance => {
        (instance).should.have.property('key').which.is.an.Number();
        (instance).should.have.property('title').which.is.an.String();
        (instance).should.have.property('episodes').which.is.an.Number();
        (instance).should.have.property('success').which.is.a.Boolean();

        if(instance.created_at) {
            (instance).should.have.property('created_at').which.is.a.Date();
            (instance).should.have.property('updated_at').which.is.a.Date();
        }

        (instance.key).should.equal(1);
        (instance.title).should.equal('this is a test');
        (instance.episodes).should.equal(4);
        (instance.success).should.be.True();
    };

    const createSample = () => {
        return {
            key: Date.now(),
            name: _.sample(['this is a test', '', null]),
            bool: _.sample([null, true, false, undefined]),
            array: _.sample([[0, 1, 2], null, []]),
            object: {a: 1, b: 2, c: 'test'},
            nullObj: null,
            date: new Date(),
            datetime: new Date(),
            timestamp: new Date(),
        };
    };

    const schema = {
        key: {
            type: Dblize.type.STRING,
            primaryKey: true,
        },
        name: {
            type: Dblize.type.STRING,
        },
        bool: {
            type: Dblize.type.BOOLEAN,
        },
        array: {
            type: Dblize.type.JSON,
        },
        object: {
            type: Dblize.type.JSON,
        },
        nullObj: {
            type: Dblize.type.STRING,
        },
        date: {
            type: Dblize.type.DATE,
            allowNull: true,
        },
        datetime: {
            type: Dblize.type.DATETIME,
            allowNull: true,
        },
        timestamp: {
            type: Dblize.type.TIMESTAMP,
            allowNull: true,
        },
    };

    const partitionSchema = {
        key: {
            type: Dblize.type.STRING,
            primaryKey: true,
        },
        name: {
            type: Dblize.type.STRING,
        },
        timestamp: {
            type: Dblize.type.TIMESTAMP,
            allowNull: true,
            partitionKey: true,
        },
    };

    /* -------------------------------------------------------------------------
     * Indexes
     * ---------------------------------------------------------------------- */

    describe('Permanent model', () => {
        let lastCount;

        it('should define a model', async () => {
            TestModel = conn.define('resource_data', schema);
        });

        it('should exists', async () => {
            let exists = await TestModel.exists();
            should(exists).be.true();
        });

        it('should create the table', async () => {
            await TestModel.sync();
        });

        it('should count rows', async () => {
            let count = await TestModel.count();
            should(count).be.above(100);

            lastCount = count;
        });

        it('should insert 1 row', async () => {
            await TestModel.create(createSample());

            // validateRecord(instance);

            let count = await TestModel.count();
            should(count).be.equal(lastCount + 1);

            lastCount = count;
        });

        it('should insert 100 row', async () => {
            let rows = [];

            for(let i = 0; i < 100; i++) {
                rows.push(createSample());
            }

            await TestModel.bulkCreate(rows);

            let count = await TestModel.count();
            should(count).be.equal(lastCount + 100);

            lastCount = count;
        });

        it('should explain', async () => {
            let explain = await TestModel.explain({
                where: {
                    name: 'this is a test',
                },
                limit: 20,
            });

            should(explain).have.property('statistics').which.is.an.Object();
        });

        it('should force maxResults', async () => {
            let rows = await TestModel.query('SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 1000', {
                maxResults: 100,
            });

            should(rows).be.length(100);
        });

        // it('should force timeout', async () => {
        //     let rows = await TestModel.query('SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 10', {
        //         timeout: 100,
        //     });
        //
        //     // should(rows).be.length(100);
        // });

        it('should force standard sql mode', async () => {
            let rows = await TestModel.query('/* useLegacySql */ SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 10', {
                useLegacySql: false,
            });

            should(rows).be.length(10);
        });

        it('should force legacy sql mode', async () => {
            let rows = await TestModel.query('/* useStandardSql */ SELECT * FROM `%NAMESPACE%.%TABLE%` LIMIT 10', {
                useLegacySql: true,
            });

            should(rows).be.length(10);
        });

        it('should query 1 row', async () => {
            let row = await TestModel.findOne({
                where: {
                    name: 'this is a test',
                },
            });

            should(row).have.property('name').which.is.equal('this is a test');
        });

        it('should query 20 rows', async () => {
            let rows = await TestModel.findAll({
                where: {
                    name: 'this is a test',
                },
                limit: 20,
            });

            should(rows).be.length(20);

            rows.forEach(row => {
                should(row).have.property('name').which.is.equal('this is a test');
            });
        });

        it('should delete all', () => {
            // return TestModel.destroy();
        });

    });

    /* -------------------------------------------------------------------------
     * Partitions
     * ---------------------------------------------------------------------- */

    describe('Permanent partition model', () => {
        it('should define a model', async () => {
            PartitionTestModel = conn.define('resource_data_partition', partitionSchema);
            await PartitionTestModel.destroy();
        });

        it('should not exists', async () => {
            let exists = await PartitionTestModel.exists();
            should(exists).be.false();
        });

        it('should create the table', async () => {
            await PartitionTestModel.sync();
        });

        it('should count rows', async () => {
            let count = await PartitionTestModel.count();
            should(count).be.equal(0);
        });

        it('should metadata', async () => {
            let metadata = await PartitionTestModel.metadata();

            should(metadata).have.a.property('timePartitioning').which.is.an.Object();
            should(metadata.timePartitioning).have.a.property('field').which.equal('timestamp');
        });

        it('should fail legacy query', () => {
            return PartitionTestModel
                .query({
                    where: {
                        name: 'this is a test',
                    },
                    limit: 20,
                })
                .should.be.rejected(/partition/i);
        });

        it('should query with standar sql', async () => {
            let rows = await PartitionTestModel.query({
                where: {
                    name: 'this is a test',
                },
                limit: 20,

            }, {
                useLegacySql: false,
            });

            should(rows).be.length(0);
        });

        it('should delete all', () => {
            return PartitionTestModel.destroy();
        });

    });
});

==> paradox-dblize-1.1.23.tgz: package/test/bigquery.js <==
'use strict';

const chai = require('chai');
const mocha = require('mocha');
const should = require('should');
const asyncp = require('paradox-asyncp');

const Dblize = require('../index');
const _ = require('lodash');
const Promise = require('bluebird');
const moment = require('moment');

describe('Bigquery', function() {
    this.timeout(10 * 1000);

    let conn = null;
    let TestModel = null;
    let PartitionTestModel = null;

    /* -------------------------------------------------------------------------
     * Before
     * ---------------------------------------------------------------------- */

    before(function() {
        conn = new Dblize({
            adapter: 'bigquery',
            credentials: {
                projectId: process.env.CRED_PROJECTID,
                keyFilename: process.env.CRED_KEYFILE,
            },
            namespace: 'test',
            sync: {
                force: false,
            }
            // forcePartition: true,
            // emptyThrowException: false,
        });
    });

    /* -------------------------------------------------------------------------
     * Create
     * ---------------------------------------------------------------------- */

    const validateRecord = instance => {
        (instance).should.have.property('key').which.is.an.Number();
        (instance).should.have.property('title').which.is.an.String();
        (instance).should.have.property('episodes').which.is.an.Number();
        (instance).should.have.property('success').which.is.a.Boolean();

        if(instance.created_at) {
            (instance).should.have.property('created_at').which.is.a.Date();
            (instance).should.have.property('updated_at').which.is.a.Date();
        }

        (instance.key).should.equal(1);
        (instance.title).should.equal('this is a test');
        (instance.episodes).should.equal(4);
        (instance.success).should.be.True();
    };

    const createSample = (recordPos) => {
        let date = new Date();

        if(!_.isUndefined(recordPos)) {
            date = moment().startOf('day').add(recordPos, 'hour').add(_.random(0, 999), 'ms').add(recordPos % 20, 'minute');

            switch(recordPos % 3) {
                case 0:
                    // no change
                    break;
                case 1:
                    date = date.toDate();
                    break;
                case 2:
                    date = date.toISOString();
                    break;
            }
        }

        return {
            key: Date.now(),
            name: _.sample(['this is a test', '', null]),
            bool: _.sample([null, true, false, undefined]),
            array: _.sample([[0, 1, 2], null, []]),
            object: {a: 1, b: 2, c: 'test'},
            nullObj: null,
            date: date,
            datetime: date,
            timestamp: date,
            repeated: [
                {
                    keyword: 'test',
                    count: 100,
                },
                {
                    keyword: 'test2',
                    count: 201
                }
            ],
            record: {
                keyword: 'record',
                count: 203
            }
        };
    };

    const schema = {
        key: {
            type: Dblize.type.STRING,
            primaryKey: true,
        },
        name: {
            type: Dblize.type.STRING,
        },
        bool: {
            type: Dblize.type.BOOLEAN,
        },
        array: {
            type: Dblize.type.JSON,
        },
        object: {
            type: Dblize.type.JSON,
        },
        nullObj: {
            type: Dblize.type.STRING,
        },
        date: {
            type: Dblize.type.DATE,
            allowNull: true,
        },
        datetime: {
            type: Dblize.type.DATETIME,
            allowNull: true,
        },
        repeated: {
            type: Dblize.type.ARRAY({
                keyword: {
                    type: Dblize.type.STRING,
                },
                count: {
                    type: Dblize.type.INTEGER,
                }
            })
        },
        record: {
            type: Dblize.type.OBJECT({
                keyword: {
                    type: Dblize.type.STRING,
                },
                count: {
                    type: Dblize.type.INTEGER,
                }
            })
        },
        timestamp: {
            type: Dblize.type.TIMESTAMP,
            allowNull: true,
        },
    };

    const partitionSchema = {
        key: {
            type: Dblize.type.STRING,
            primaryKey: true,
        },
        name: {
            type: Dblize.type.STRING,
        },
        timestamp: {
            type: Dblize.type.TIMESTAMP,
            allowNull: true,
            partitionKey: true,
        },
    };

    /* -------------------------------------------------------------------------
     * Indexes
     * ---------------------------------------------------------------------- */

    describe('Permanent model', () => {
        let lastCount;

        it('should define a model', async () => {
            TestModel = conn.define('resource_data', schema);
        });

        it('should exists', async () => {
            let exists = await TestModel.exists();
            should(exists).be.true();
        });

        it('should create the table', async () => {
            await TestModel.sync();
        });

        it('should count rows', async () => {
            let count = await TestModel.count();
            should(count).be.above(100);

            lastCount = count;
        });

        it('should insert 1 row', async () => {
            await TestModel.create(createSample());

            // validateRecord(instance);

            let count = await TestModel.count();
            should(count).be.equal(lastCount + 1);

            lastCount = count;
        });

        it('should insert 100 row', async () => {
            let rows = [];

            for(let i = 0; i < 100; i++) {
                rows.push(createSample(i));
            }

            await TestModel.bulkCreate(rows);

            let count = await TestModel.count();
            should(count).be.equal(lastCount + 100);

            lastCount = count;
        });

        it('should explain', async () => {
            let explain = await TestModel.explain({
                where: {
                    name: 'this is a test',
                },
                limit: 20,
            });

            should(explain).have.property('statistics').which.is.an.Object();
        });

        it('should force maxResults', async () => {
            let rows = await TestModel.query('SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 1000', {
                maxResults: 100,
            });

            should(rows).be.length(100);
        });

        // it('should force timeout', async () => {
        //     let rows = await TestModel.query('SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 10', {
        //         timeout: 100,
        //     });
        //
        //     // should(rows).be.length(100);
        // });

        it('should force standard sql mode', async () => {
            let rows = await TestModel.query('/* useLegacySql */ SELECT * FROM [%NAMESPACE%.%TABLE%] LIMIT 10', {
                useLegacySql: false,
            });

            should(rows).be.length(20); // 10 + 10 repeated
        });

        it('should force legacy sql mode', async () => {
            let rows = await TestModel.query('/* useStandardSql */ SELECT * FROM `%NAMESPACE%.%TABLE%` LIMIT 10', {
                useLegacySql: true,
            });

            should(rows).be.length(10);
        });

        it('should query 1 row', async () => {
            let row = await TestModel.findOne({
                where: {
                    name: 'this is a test',
                },
            });

            should(row).have.property('name').which.is.equal('this is a test');
        });

        it('should query 20 rows', async () => {
            let rows = await TestModel.findAll({
                where: {
                    name: 'this is a test',
                },
                limit: 20,
            });

            should(rows).be.length(40); // 20 + 20 repeated

            rows.forEach(row => {
                should(row).have.property('name').which.is.equal('this is a test');
            });
        });

        it('should delete all', () => {
            // return TestModel.destroy();
        });

    });

    /* -------------------------------------------------------------------------
     * Partitions
     * ---------------------------------------------------------------------- */

    describe('Permanent partition model', () => {
        it('should define a model', async () => {
            PartitionTestModel = conn.define('resource_data_partition', partitionSchema);
            await PartitionTestModel.destroy();
        });

        it('should not exists', async () => {
            let exists = await PartitionTestModel.exists();
            should(exists).be.false();
        });

        it('should create the table', async () => {
            await PartitionTestModel.sync();
        });

        it('should count rows', async () => {
            let count = await PartitionTestModel.count();
            should(count).be.equal(0);
        });

        it('should metadata', async () => {
            let metadata = await PartitionTestModel.metadata();

            should(metadata).have.a.property('timePartitioning').which.is.an.Object();
            should(metadata.timePartitioning).have.a.property('field').which.equal('timestamp');
        });

        it('should fail legacy query', () => {
            return PartitionTestModel
                .query({
                    where: {
                        name: 'this is a test',
                    },
                    limit: 20,
                })
                .should.be.rejected(/partition/i);
        });

        it('should query with standar sql', async () => {
            let rows = await PartitionTestModel.query({
                where: {
                    name: 'this is a test',
                },
                limit: 20,

            }, {
                useLegacySql: false,
            });

            should(rows).be.length(0);
        });

        it('should delete all', () => {
            return PartitionTestModel.destroy();
        });

    });
});
